@@ -1,7 +1,7 @@
 """
 This **module** gives the basic functions for fep calculations
 """
-from ...helper import source, set_global_alternative_names, Xdict
+from ...helper import source, set_global_alternative_names, Xdict, Xopen, Xprint
 from ..base import lj_base, exclude_base, bond_base, angle_base, dihedral_base, nb14_extra_base
 
 source("....")
@@ -96,7 +96,7 @@ def _nb14_extra_merge_rule(mol_r, mol_a, mol_b, forcetype, rforces, bforces, lam
 
             f_r.A = f_b.A * lambda_
             f_r.B = f_b.B * lambda_
-            f_r.kee = f2.kee * lambda_
+            f_r.kee = f_b.kee * lambda_
             mol_r.Add_Bonded_Force(f_r)
         else:
             temp_charge0 = f_r.atoms[0].charge if abs(f_r.atoms[0].charge) > TINY else TINY
@@ -605,7 +605,8 @@ def _get_residue_ab(residue_type_a, residue_type_b, residue_a, forcopy, matchmap
     return restype_ab, rbmap
 
 
-def merge_dual_topology(mol, residue_a, residue_b, assign_a, assign_b, tmcs=60):
+def merge_dual_topology(mol, residue_a, residue_b, assign_a, assign_b,
+                        tmcs=60, image_path=None, similarity_limit=0):
     """
     This **function** perturbs a residue in the molecule into another type in the dual topology way
 
@@ -615,6 +616,9 @@ def merge_dual_topology(mol, residue_a, residue_b, assign_a, assign_b, tmcs=60):
     :param assign_a: the Assign instance corresponding to ``residue_a``
     :param assign_b: the Assign instance corresponding to ``residue_b``
     :param tmcs: the max time to find the max common structure
+    :param image_path: the path to save the mcs image
+    :param similarity_limit: the limitation of the similarity. The similarity is calculated by \
+the tanimoto coefficient of the max common structure.
     :return: two molecules in the initial and final lambda stat respectively
     """
     build.Build_Bonded_Force(mol)
@@ -622,6 +626,7 @@ def merge_dual_topology(mol, residue_a, residue_b, assign_a, assign_b, tmcs=60):
 
     from ...helper.rdkit import assign_to_rdmol, insert_atom_type_to_rdmol
     from rdkit.Chem import rdFMCS as MCS
+    from rdkit.Chem import Draw, AllChem
 
     rdmol_a = assign_to_rdmol(assign_a, True)
     rdmol_b = assign_to_rdmol(assign_b, True)
@@ -629,7 +634,7 @@ def merge_dual_topology(mol, residue_a, residue_b, assign_a, assign_b, tmcs=60):
     atom_type_dict = Xdict()
     insert_atom_type_to_rdmol(rdmol_a, residue_a, assign_a, atom_type_dict)
     insert_atom_type_to_rdmol(rdmol_b, residue_b, assign_b, atom_type_dict)
-    print("FINDING MAXIMUM COMMON SUBSTRUCTURE")
+    print("FINDING MAXIMUM COMMON SUBSTRUCTURE\n")
 
     result = MCS.FindMCS([rdmol_a, rdmol_b], atomCompare=MCS.AtomCompare.CompareIsotopes, completeRingsOnly=True,
                          timeout=tmcs)
@@ -637,9 +642,32 @@ def merge_dual_topology(mol, residue_a, residue_b, assign_a, assign_b, tmcs=60):
 
     match_a = rdmol_a.GetSubstructMatch(rdmol_mcs)
     match_b = rdmol_b.GetSubstructMatch(rdmol_mcs)
+    tanimoto = len(match_a)
+    tanimoto /= len(assign_a.atoms) + len(assign_b.atoms) - tanimoto
+    Xprint(f"similarity: {tanimoto}")
+    assert tanimoto > similarity_limit, f"similarity (={tanimoto}) should be greater than {similarity_limit}"
     matchmap = {match_b[j]: match_a[j] for j in range(len(match_a))}
+    if image_path:
+        draw_a = assign_to_rdmol(assign_a, True)
+        draw_b = assign_to_rdmol(assign_b, True)
+        AllChem.Compute2DCoords(draw_a)
+        AllChem.Compute2DCoords(draw_b)
+        for atom in draw_a.GetAtoms():
+            atom.SetProp("atomLabel", atom.GetSymbol())
+        for atom in draw_b.GetAtoms():
+            atom.SetProp("atomLabel", atom.GetSymbol())
+        img = Draw.MolsToGridImage([draw_a, draw_b],
+                                   molsPerRow=1,
+                                   subImgSize=(1200, 600),
+                                   highlightAtomLists=[match_a, match_b])
+        img.save(image_path)
+        f = Xopen(os.path.splitext(image_path)[0] + ".txt", "w")
+        f.write(f"similarity: {tanimoto}\n")
+        f.write(f"mcs atoms in r1: {match_a}\n\n")
+        f.write(f"mcs atoms in r2: {match_b}\n\n")
+        f.close()
 
-    print("ALIGNING TOPOLOGY AND COORDINATE")
+    print("ALIGNING TOPOLOGY AND COORDINATE\n")
 
     residue_type_a = residue_a.type
 
