@@ -5,10 +5,10 @@ import os
 import sys
 import unittest
 import multiprocessing as mpc
+
 from ..helper import source, GlobalSetting, Xopen, Xprint
 from ..mdrun import run
 
-
 class TestMyPackage(unittest.TestCase):
     """
     This **class** does the unit tests for Xponge
@@ -183,7 +183,7 @@ class TestMyPackage(unittest.TestCase):
         Save_Mol2(NALA, f"{args.o}_r1.mol2")
         Save_Mol2(NGLY, f"{args.o}_r2.mol2")
         error = os.system(f"Xponge mol2rfe -nl 1 -pdb {args.o}.pdb -r1 {args.o}_r1.mol2 \
--r2 {args.o}_r2.mol2 -pstep 1000 -estep 1000 > {os.devnull}")
+-r2 {args.o}_r2.mol2 -pstep 1000 -esteps 1000 > {os.devnull}")
         self.assertEqual(error, 0)
 
 
@@ -480,8 +480,7 @@ def _mol2rfe_build(args, merged_from, merged_to):
     :return:
     """
     source("..")
-    fep = source("..forcefield.special.fep")
-    min_ = source("..forcefield.special.min")
+    fep = source("..forcefield.special.fep", False)
 
     if "build" in args.do:
         Xprint("\nBUILDING TOPOLOGY\n", verbose=-1)
@@ -493,11 +492,17 @@ def _mol2rfe_build(args, merged_from, merged_to):
             os.mkdir("%d" % i)
             tt = fep.Merge_Force_Field(merged_from, merged_to, i / args.nl)
             if i == 0:
-                min_.save_min_bonded_parameters()
-            elif i == 1:
-                min_.do_not_save_min_bonded_parameters()
+                Xprint("Initial Structure Optimizing\n")
+                process.optimize(tt,
+                                 only_bad_coordinate=False,
+                                 extra_commands={"lambda_lj": i / args.nl})
+                for atom_m, atom_t in zip(merged_from.atoms, tt.atoms):
+                    atom_m.x = atom_t.x
+                    atom_m.y = atom_t.y
+                    atom_m.z = atom_t.z
+            build.save_mol2(tt, "%d/%s.mol2" % (i, args.temp))
             build.Save_SPONGE_Input(tt, "%d/%s" % (i, args.temp))
-
+            Xprint(f"{i} built success")
 
 def _mol2rfe_output_path(subdir, workdir, tempname):
     """
@@ -530,7 +535,7 @@ def _mol2rfe_min(args):
             basic += f" -mode minimization -lambda_lj {lambda_}"
             basic += _mol2rfe_output_path("min", i, args.temp)
             if not args.mi:
-                cif = " -constrain_mode SHAKE"
+                cif = " -constrain_mode SHAKE -cutoff 8"
                 run(f"{basic} {cif} -dt 1e-8 -step_limit {args.msteps[0]}")
                 cif += " -coordinate_in_file {1}/min/{0}_coordinate.txt".format(args.temp, i)
                 run(f"{basic} {cif} -dt 1e-7 -step_limit {args.msteps[1]}")
@@ -562,7 +567,7 @@ def _mol2rfe_pre_equilibrium(args):
             os.mkdir("%d/pre_equilibrium" % i)
             command = f"SPONGE -default_in_file_prefix {i}/{args.temp}"
             lambda_ = i / args.nl
-            command += f" -lambda_lj {lambda_}"
+            command += f" -lambda_lj {lambda_} -cutoff 8"
             command += _mol2rfe_output_path("pre_equilibrium", i, args.temp)
             command += f" -coordinate_in_file {i}/min/{args.temp}_coordinate.txt"
             if not args.pi:
@@ -589,69 +594,61 @@ def _mol2rfe_equilibrium(args):
             os.mkdir("%d/equilibrium" % i)
             command = f"SPONGE -default_in_file_prefix {i}/{args.temp}"
             lambda_ = i / args.nl
-            command += f" -lambda_lj {lambda_}"
+            command += f" -lambda_lj {lambda_} -cutoff 8"
             command += _mol2rfe_output_path("equilibrium", i, args.temp)
             command += f" -coordinate_in_file {i}/pre_equilibrium/{args.temp}_coordinate.txt"
             if not args.ei:
                 command += f" -mode NPT -step_limit {args.equilibrium_step} -dt {args.dt} -constrain_mode SHAKE"
                 command += f" -barostat {args.barostat} -thermostat {args.thermostat}"
+                command += f" -write_information_interval 100 -write_restart_file_interval {args.equilibrium_step}"
                 run(command)
             else:
                 command += f" -mdin {args.pi}"
                 run(command)
 
 
-def _mol2rfe_analysis(args, merged_from):
+def _mol2rfe_analysis(args, merged_from, merged_to):
     """
 
     :param args:
     :param merged_from:
+    :param merged_to:
     :return:
     """
     source("..")
+    source("..analysis")
 
     if "analysis" in args.do:
         f = Xopen("dh_dlambda.txt", "w")
-        f.write("")
         f.close()
+        resname = merged_from.residues[args.ri].name
+        draw_r1_mol = merged_from.deepcopy()
+        draw_r2_mol = merged_to.deepcopy()
+        load_coordinate(f"0/equilibrium/{args.temp}_coordinate.txt", draw_r1_mol)
+        load_coordinate(f"{args.nl}/equilibrium/{args.temp}_coordinate.txt", draw_r2_mol)
+        draw_r1_res = draw_r1_mol.residues[args.ri]
+        draw_r2_res = draw_r2_mol.residues[args.ri]
+        draw_r1_res.name = resname.split("_")[0]
+        draw_r2_res.name = resname.split("_")[1]
+        to_delete = []
+        for atom in draw_r1_res.atoms:
+            if atom.LJtype == "ZERO_LJ_ATOM":
+                to_delete.append(atom)
+        for atom in to_delete:
+            draw_r1_res.atoms.remove(atom)
+        to_delete = []
+        for atom in draw_r2_res.atoms:
+            if atom.LJtype == "ZERO_LJ_ATOM":
+                to_delete.append(atom)
+            if atom.name.endswith("R2"):
+                atom.name = atom.name[:-2]
+        for atom in to_delete:
+            draw_r2_res.atoms.remove(atom)
+        save_pdb(draw_r1_mol, "r1.pdb")
+        save_pdb(draw_r2_mol, "r2.pdb")
         if args.method == "TI":
-            for i in range(args.nl + 1):
-                if os.path.exists("%d/ti" % i):
-                    os.system("rm -rf %d/ti" % i)
-                os.mkdir("%d/ti" % i)
-                inprefix = f"{i}/{args.temp}"
-                command = f"SPONGE_TI -LJ_soft_core_in_file {inprefix}_LJ_soft_core.txt"
-                command += " -exclude_in_file {0}_exclude.txt -charge_in_file {0}_charge.txt".format(inprefix)
-                command += f" -chargeA_in_file 0/{args.temp}_charge.txt"
-                command += f" -chargeB_in_file {args.nl}/{args.temp}_charge.txt"
-                lambda_ = i / args.nl
-                command += f" -lambda_lj {lambda_}"
-                command += f" -subsys_division_in_file {inprefix}_subsys_division.txt  -charge_pertubated 1"
-                inprefix = f"{i}/ti/{args.temp}"
-                command += f" -mdinfo {inprefix}.mdinfo -mdout {inprefix}.mdout"
-                inprefix = f"{i}/equilibrium/{args.temp}"
-                command += f" -crd {inprefix}.dat -box {inprefix}.box -TI dh_dlambda.txt"
-                command += f" -atom_numbers {len(merged_from.atoms)}"
-                command += f" -frame_numbers {args.equilibrium_step // 100}"
-                if not args.ai:
-                    run(command)
-                else:
-                    command += f" -mdin {args.ai}"
-                    run(command)
-            dh_dlambda = np.loadtxt("dh_dlambda.txt")
-            dh = []
-            dh_int = []
-            tempall = 0
-            for i in range(args.nl):
-                temp = dh_dlambda[i] * 0.5 / args.nl
-                temp += dh_dlambda[i + 1] * 0.5 / args.nl
-                dh.append(temp)
-                tempall += temp
-                dh_int.append(tempall)
-            f = Xopen("free_energy.txt", "w")
-            f.write("lambda_state\tFE(i+1)-FE(i)[kcal/mol]\tFE(i+1)-FE(0)[kcal/mol]\n")
-            f.write("\n".join(["%d\t\t%.2f\t\t\t%.2f" % (i, dh[i], dh_int[i]) for i in range(args.nl)]))
-            f.close()
+            ti = source(".ti", False)
+            ti.ti_analysis(args, merged_from)
         elif args.method == "FEP_BAR":
             raise NotImplementedError
 
@@ -697,7 +694,9 @@ def mol2rfe(args):
 
     rmol = load_pdb(args.pdb)
 
-    merged_from, merged_to = Merge_Dual_Topology(rmol, rmol.residues[args.ri], to_res_type_, from_, to_, args.tmcs)
+    merged_from, merged_to = Merge_Dual_Topology(rmol, rmol.residues[args.ri],
+                                                 to_res_type_, from_, to_,
+                                                 args.tmcs, f"{args.fmcs}", args.lmcs)
 
     if args.dohmr:
         H_Mass_Repartition(merged_from)
@@ -711,4 +710,4 @@ def mol2rfe(args):
 
     _mol2rfe_equilibrium(args)
 
-    _mol2rfe_analysis(args, merged_from)
+    _mol2rfe_analysis(args, merged_from, merged_to)
